﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    public partial class HbmColumn
    {
        [System.Xml.Serialization.XmlIgnoreAttribute()]
        internal String table = null;
        // default
        public HbmColumn()
        { 
        }
        
        public HbmColumn(Column column, PropertyHelper propery)
        {
            apply(column);
            cache(propery);
        }

        private void cache(PropertyHelper propery)
        {
            if (propery == null)
            {
                return;
            }
            MappingCache.getInstance().registerColumn(propery.getPath(), this);
        }

        internal void apply(Column column)
        {
            this.name = column.Name;
            if (column.Length !=255)// apply only if different than difault
            {
                this.length = column.Length.ToString();
            }
            if (column.Precision > 0)
            {
                this.precision = column.Precision.ToString();
            }
            if (column.Scale > 0)
            {
                this.scale = column.Scale.ToString();
            }
            if (!string.IsNullOrEmpty(column.ColumnDefinition))
            {
                this.sqltype = column.ColumnDefinition;
            }

            if (!column.Nullable)
            {
                this.notnull = true;
                this.notnullSpecified = true;
            }

            if (column.Unique)
            {
                this.unique = true;
                this.uniqueSpecified = true;
            }
            this.table = column.Table;
            // handled elsewhere
            //this.index 
            // this.uniqueKey

            //Not supported:
            //this.check
            //this.@default 
            //column.Insertable
            //column.Updated
            
        }

        public HbmColumn(MapKeyColumn column)
        {
            this.name = column.Name;
            if (column.Length != 255)// apply only if different than default
            {
                this.length = column.Length.ToString();
            }
            if (column.Precision > 0)
            {
                this.precision = column.Precision.ToString();
            }
            if (column.Scale > 0)
            {
                this.scale = column.Scale.ToString();
            }
            if (!string.IsNullOrEmpty(column.ColumnDefinition))
            {
                this.sqltype = column.ColumnDefinition;
            }

            if (!column.Nullable)
            {
                this.notnull = true;
                this.notnullSpecified = true;
            }

            if (column.Unique)
            {
                this.unique = true;
                this.uniqueSpecified = true;
            }
            this.table = column.Table;
        }


        public HbmColumn(MapKeyJoinColumn column)
        {
            this.name = column.Name;
            if (!column.Nullable)
            {
                this.notnull = true;
                this.notnullSpecified = true;
            }
            if (column.Unique)
            {
                this.unique = true;
                this.uniqueSpecified = true;
            }
            if (!string.IsNullOrEmpty(column.ColumnDefinition))
            {
                this.sqltype = column.ColumnDefinition;
            }
            if (!String.IsNullOrEmpty(column.ReferencedColumnName))
            {
                throw new NotImplementedException("MapKeyJoinColumn.ReferencedColumnName is not implemented yet");
            }
            this.table = column.Table;
        }

        public HbmColumn(JoinColumn column)
        {
            this.name = column.Name;
            if (!column.Nullable)
            {
                this.notnull = true;
                this.notnullSpecified = true;
            }
            if (column.Unique)
            {
                this.unique = true;
                this.uniqueSpecified = true;
            }
            if (!string.IsNullOrEmpty(column.ColumnDefinition))
            {
                this.sqltype = column.ColumnDefinition;
            }
            this.table = column.Table;
            //Not supported:
            //this.index 
            //this.check
            //this.@default 
            //column.Insertable
            //column.Updated
            //column.table
            
        }

        public HbmColumn(OrderColumn column)
        {
            
            this.name = column.Name;
            if (!column.Nullable)
            {
                this.notnull = true;
                this.notnullSpecified = true;
            }
            if (!string.IsNullOrEmpty(column.ColumnDefinition))
            {
                this.sqltype = column.ColumnDefinition;
            }
            
            //Unique-key is not handled since the way to 
            // define a unique key in JPA is in the @Table annotation.
            //this.uniquekey

            //Not supported:
            //this.index 
            //this.check
            //this.@default 
            //column.Insertable
            //column.Updated
            //column.table

        }
    }
}
