using System.Collections.Generic;
using System.Xml.Serialization;
using System;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmDiscriminator
	{
        public HbmDiscriminator(DiscriminatorColumn discriminatorCol)
            : this()
        {
            if (discriminatorCol == null)
            {
                return;
            }
            this.column = discriminatorCol.Name;
            this.length = discriminatorCol.Length.ToString();
            this.type = parseDiscriminatorType(discriminatorCol.DiscriminatorType);
        }

        private string parseDiscriminatorType(DiscriminatorType discriminatorType)
        {
            if (discriminatorType == DiscriminatorType.INTEGER)
            {
                return typeof(int).FullName;
            }
            if (discriminatorType == DiscriminatorType.CHAR)
            {
                return typeof(char).FullName;
            }
            if (discriminatorType == DiscriminatorType.STRING)
            {
                return typeof(String).FullName;
            }
            throw new Exception("Unknown discriminator type <" + discriminatorType + ">.");
        }


		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get
			{
				if(Item as HbmColumn != null)
				{
					yield return (HbmColumn)Item;
				}
				else if (string.IsNullOrEmpty(column))
				{
					yield break;
				}
				else
				{
					yield return new HbmColumn
					{
						name = column,
						length = length,
						notnull = notnull,
						notnullSpecified = true
					};
				}
			}
		}

		#endregion

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get
			{
				if (Item as HbmFormula != null)
				{
					yield return (HbmFormula) Item;
				}
				else if (string.IsNullOrEmpty(formula))
				{
					yield break;
				}
				else
				{
					yield return new HbmFormula {Text = new[] {formula}};
				}
			}
		}

		#endregion
	}
}