﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
    public partial class HbmGenerator
    {
        public HbmGenerator(GeneratedValue generatedValue, object generator)
            : this()
        {
            GenerationType genType = generatedValue.Strategy;
            if (genType == GenerationType.TABLE)
            {
                createTableGenerator(generatedValue, generator);
            }
            else if (genType == GenerationType.SEQUENCE)
            {
                createSequenceGenerator(generatedValue, generator);
            }
            else if (genType == GenerationType.IDENTITY)
            {
                this.@class = "identity";
            }
            else//AUTO
            {
                this.@class = "native";
            }
        }

        private void createSequenceGenerator(GeneratedValue generatedValue, object generator)
        {
            if (!(generator is SequenceGenerator))
            {
                throw new PersistenceException("SequenceGenerator is expected. got " + generator);
            }
            SequenceGenerator sequenceGen = (SequenceGenerator)generator;
            if (!generatedValue.Generator.Equals(sequenceGen.Name))
            {
                throw new PersistenceException("Wrong generator. Expected: " + generatedValue.Generator + " got: " + sequenceGen.Name);
            }

            //this.@class = "NHibernate.Id.Enhanced.SequenceStyleGenerator, NHibernate";
            this.@class = "seqhilo";
            this.param = new HbmParam[6];
            int i = 0;
            if (sequenceGen.SequenceName != null && sequenceGen.SequenceName.Length > 0)
            {
                //param[i] = new HbmParam() { name = "sequence_name", Text = new String[] { sequenceGen.SequenceName } };
                param[i] = new HbmParam() { name = "sequence", Text = new String[] { sequenceGen.SequenceName } };
                i++;
            }
            if (!String.IsNullOrEmpty(sequenceGen.Schema))
            {
                param[i] = new HbmParam() { name = "schema", Text = new String[] { sequenceGen.Schema } };
                i++;
            }
            if (sequenceGen.InitialValue > 1)
            {
                //param[i] = new HbmParam() { name = "initial_value", Text = new String[] { sequenceGen.InitialValue.ToString() } };
                //i++;
                throw new NotSupportedException("InitialValue for sequence generator is not supported by NHibernate");
            }
            //param[i] = new HbmParam() { name = "increment_size", Text = new String[] { sequenceGen.AllocationSize.ToString() } };
            param[i] = new HbmParam() { name = "max_lo", Text = new String[] { sequenceGen.AllocationSize.ToString() } };
        }

        private void createTableGenerator(GeneratedValue generatedValue, object generator)
        {
            if (!(generator is TableGenerator))
            {
                throw new PersistenceException("TableGenerator is expected. got " + generator);
            }
            TableGenerator tableGen = (TableGenerator)generator;
            if (!generatedValue.Generator.Equals(tableGen.Name))
            {
                throw new PersistenceException("Wrong generator. Expected: " + generatedValue.Generator + " got: " + tableGen.Name);
            }
            this.@class = "hilo";
            this.param = new HbmParam[6];
            int i = 0;
            if (!String.IsNullOrEmpty(tableGen.Table))
            {
                param[i] = new HbmParam() { name = "table", Text = new String[] { tableGen.Table } };
                i++;
            }
            if (!String.IsNullOrEmpty(tableGen.ValueColumnName))
            {
                param[i] = new HbmParam() { name = "column", Text = new String[]{tableGen.ValueColumnName}};
                i++;
            }
            if (!String.IsNullOrEmpty(tableGen.Schema))
            {
                param[i] = new HbmParam() { name = "schema", Text = new String[] { tableGen.Schema } };
                i++;
            }
            if (!String.IsNullOrEmpty(tableGen.PkColumnName))
            {
                throw new NotSupportedException("PkColumnName for table generator is not supported by NHibernate");
            }
            if (!String.IsNullOrEmpty(tableGen.PkColumnValue))
            {
                throw new NotSupportedException("PkColumnValue for table generator is not supported by NHibernate");
            }
            if (tableGen.InitialValue > 0)
            {
                throw new NotSupportedException("InitialValue for table generator is not supported by NHibernate");
            }

            param[i] = new HbmParam() { name = "max_lo", Text = new String[] { tableGen.AllocationSize.ToString() } };
            
        }
    }
}
