using System.Collections.Generic;
using System.Xml.Serialization;
using System;
using System.Reflection;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmKey
	{
        private bool isManyToMany;
        private bool isInverse;
        public HbmKey(JoinTableHelper joinTable, Type holdingEntity, bool isManyToMany, bool isInverse):this()
        {
            this.isManyToMany = isManyToMany;
            this.isInverse = isInverse;
            handleJoinTable(joinTable, holdingEntity);
        }

        public HbmKey(JoinColumn[] joinColumns, Type holdingEntity, bool isManyToMany, bool isInverse):this()
        {
            this.isManyToMany = isManyToMany;
            this.isInverse = isInverse;
            if (!(isManyToMany && isInverse))
            {
                this.column = MappingUtils.getJoinColumns(joinColumns, holdingEntity, ref this.propertyref);
            }
        }

        public HbmKey(CollectionTableHelper collectionTable, Type holdingEntity):this()
        {
            this.column = MappingUtils.getJoinColumns(collectionTable.joinColumns, holdingEntity, ref this.propertyref);
        }

        public HbmKey(PrimaryKeyJoinColumn[] keyColArr, Type holdingEntity)
            : this()
        {
            this.column = new HbmColumn[keyColArr.Length] ;
            int i = 0;
            foreach (PrimaryKeyJoinColumn joinCol in keyColArr)
            {
                HbmColumn col = new HbmColumn();
                col.name = joinCol.Name;
                this.column[i] = col;
                MappingUtils.handlePropertyRef(joinCol.ReferencedColumnName, holdingEntity, ref this.propertyref);
                i++;
            }
        }

        private void handleJoinTable(JoinTableHelper joinTable, Type holdingEntity)
        {
            if (isManyToMany && isInverse)
            {
                this.column = MappingUtils.getJoinColumns(joinTable.inverseJoinColumns, holdingEntity, ref this.propertyref);
            }
            else
            {
                this.column = MappingUtils.getJoinColumns(joinTable.joinColumns, holdingEntity, ref this.propertyref);
            }
        }

		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get { return column ?? AsColumns(); }
		}

		#endregion

		private IEnumerable<HbmColumn> AsColumns()
		{
			if (string.IsNullOrEmpty(column1))
			{
				yield break;
			}
			else
			{
				yield return new HbmColumn
				{
					name = column1,
					notnull = notnull,
					notnullSpecified = notnullSpecified,
					unique = unique,
					uniqueSpecified = uniqueSpecified,
				};
			}
		}

		public bool? IsNullable
		{
			get { return notnullSpecified ? !notnull : (bool?)null; }
		}

		public bool? IsUpdatable
		{
			get { return updateSpecified ? update : (bool?)null; }
		}
	}
}