using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using System.Reflection;
using NPersistence.Ext;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmManyToOne : TableResident, RelationObject
	{
        
        [System.Xml.Serialization.XmlIgnoreAttribute()]
        Type targetType;
        public HbmManyToOne(PropertyHelper propertyInfo)
            : this()
        {
            //configuration
            this.name = propertyInfo.Name;
            this.targetType = propertyInfo.PropertyType;
            this.@class = MappingUtils.getStringClassDef(targetType);
            
            List<Object> itemList = new List<object>();
            bool isOneToOne = false;
            bool isOptional = true;

            ManyToOne manyTo1 = propertyInfo.getSingleAttribute<ManyToOne>();
            if (manyTo1 != null)
            {
                this.cascade = CascadeTypeMapping.mapCascadeType(manyTo1.Cascade , manyTo1);
                if (manyTo1.Fetch == FetchType.EAGER)
                {
                    this.lazy = HbmLaziness.False;
                }
                if (manyTo1.TargetEntity != null && manyTo1.TargetEntity != typeof(void))
                {
                    this.@class = MappingUtils.getStringClassDef(manyTo1.TargetEntity);
                    this.targetType = manyTo1.TargetEntity;
                }
                if (!manyTo1.Optional)
                {
                    this.notnull = true;
                    this.notnullSpecified = true;
                    isOptional = false;
                }
                //For bidirectional relation and the other side is an indexed collection,
                //it is not supported by NHibernate.
                //In this case, we put here insert=false and update=false.
                //the other side of the relation is responsible for the updates.
                RelationObject mappingRelation = MappingCache.getInstance().getMappedByRelation(propertyInfo.ReflectedType, propertyInfo.getPath());
                if (mappingRelation != null && mappingRelation.isIndexed())
                {
                    this.update = false;
                    this.insert = false;
                }
                MappingCache.getInstance().registerRelation(propertyInfo.getPath(), propertyInfo.ReflectedType, this);
            }
            // Many-to-one with unique=true is one-to-one
            OneToOne oneTo1 = propertyInfo.getSingleAttribute<OneToOne>();
            if (oneTo1 != null)
            {
                isOneToOne = true;
                this.unique = true;
                this.cascade = CascadeTypeMapping.mapCascadeType(oneTo1.Cascade, oneTo1);
                if (oneTo1.Fetch == FetchType.EAGER)
                {
                    this.lazy = HbmLaziness.False;
                }
                if (oneTo1.TargetEntity != null && oneTo1.TargetEntity != typeof(void))
                {
                    this.@class = MappingUtils.getStringClassDef(oneTo1.TargetEntity);
                    this.targetType = oneTo1.TargetEntity;
                }
                if (!oneTo1.Optional)
                {
                    this.notnull = true;
                    this.notnullSpecified = true;
                    isOptional = false;
                }
            }
            JoinTableHelper joinT = MappingUtils.getJoinTable(propertyInfo, null, null);
            if (joinT != null)
            {
                // Join talbe is not supported yet
                // in order to support this, we need to create a new class which
                // will be the join class
                throw new NotSupportedException("JoinTable in ManyToOne relationship is not supported yet.");
            }
            
            JoinColumn[] joinColArr = MappingUtils.getJoinColumn(propertyInfo);
            if (joinColArr != null && joinColArr.Length > 0)
            {
                //Multiple join clumns in manyToOne is only supported for composite-id 
                // In order to support this, we need to go to the referenced class 
                // and add a 'properties' tag with all the join clumns, give the properties tag a name 
                // and set property-ref to that name.
                for (int i = 0; i < joinColArr.Length; i++)
                {
                    joinColArr[i] = MappingUtils.setJoinColumnDefaults(joinColArr[i], propertyInfo, i);
                }
                foreach (JoinColumn joinCol in joinColArr)
                {
                    handleJoinColumn(joinCol, itemList, isOptional, isOneToOne, propertyInfo);
                }
            }
            else
            {
                // create a default join column
                JoinColumn joinCol = MappingUtils.setJoinColumnDefaults(null, propertyInfo, 0);
                handleJoinColumn(joinCol, itemList, isOptional, isOneToOne, propertyInfo);
            }
            this.Items = itemList.ToArray();
        }

        private void handleJoinColumn(JoinColumn joinCol, List<Object> itemList, bool isOptional, bool isOneToOne, PropertyHelper propertyInfo)
        {
            HbmColumn col = new HbmColumn(joinCol);
            if (isOneToOne)
            {
                col.unique = true;
                col.uniqueSpecified = true;
            }
            if(!isOptional) 
            {
                col.notnull = true;
                col.notnullSpecified = true;
            }
            //handle unique constraint
            PropertyUtils.handleUniqueConstrains(propertyInfo, col);
            itemList.Add(col);
            if (!String.IsNullOrEmpty(joinCol.ReferencedColumnName))
            {
                MappingUtils.handlePropertyRef(joinCol.ReferencedColumnName, this.targetType, ref this.propertyref);
            }
        }

        public String getTable()
        {
            if (this.Items != null)
            {
                foreach (Object item in this.Items)
                {
                    if (item is HbmColumn)
                    {
                        return ((HbmColumn)this.Items[0]).table;
                    }
                }
            }
            return null;
        }

        public bool isIndexed()
        {
            return false;
        }

        public String getMappedBy()
        {
            return null;
        }

        public Type getTargetType()
        {
            return targetType;
        }
        
		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get { return Items != null ? Items.OfType<HbmColumn>() : AsColumns(); }
		}

		#endregion

		private IEnumerable<HbmColumn> AsColumns()
		{
			if (string.IsNullOrEmpty(column))
			{
				yield break;
			}
			else
			{
				yield return new HbmColumn
				{
					name = column,
					notnull = notnull,
					notnullSpecified = notnullSpecified,
					unique = unique,
					uniqueSpecified = true,
					uniquekey = uniquekey,
					index = index
				};
			}
		}

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get { return Items != null ? Items.OfType<HbmFormula>() : AsFormulas(); }
		}

		private IEnumerable<HbmFormula> AsFormulas()
		{
			if (string.IsNullOrEmpty(formula))
			{
				yield break;
			}
			else
			{
				yield return new HbmFormula { Text = new[] { formula } };
			}
		}

		#endregion

		#region Implementation of IRelationship

		public string EntityName
		{
			get { return entityname; }
		}

		public string Class
		{
			get { return @class; }
		}

		public HbmNotFoundMode NotFoundMode
		{
			get { return notfound; }
		}

		#endregion

		/// <summary>
		/// Columns and Formulas, in declared order
		/// </summary>
		[XmlIgnore]
		public IEnumerable<object> ColumnsAndFormulas
		{
			// when Items is empty the column attribute AND formula attribute will be used
			// and it may cause an issue (breaking change)
			// On the other hand it work properly when a mixing between <formula> and <column> tags are used
			// respecting the order used in the mapping to map multi-columns id.
			get { return Items ?? Columns.Cast<object>().Concat(Formulas.Cast<object>()); }
		}
		
		public HbmLaziness? Lazy
		{
			get { return lazySpecified ? lazy : (HbmLaziness?) null;}
		}
	}
}