using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using System.Reflection;
using NHibernate.Dialect;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmMapKey
	{
        // used only for simple types
        public HbmMapKey(PropertyHelper propertyInfo, System.Type mapKeyClass, System.Type mapValueClass, string mapKeyName)
            : this()
        {
            HbmColumn hbmCol;
            MapKeyColumn[] colArr = propertyInfo.getAttributeArr<MapKeyColumn>();
            if (colArr != null && colArr.Length > 0)
            {
                if (colArr.Length > 1)
                {
                    throw new PersistenceException("Multiple 'MapKeyColumn' attributes were set to a simple type");
                }
                hbmCol = new HbmColumn(colArr[0]);
            }
            else
            {
                if (mapKeyName != null)
                {
                    Column col = MappingUtils.findSingleColumn(mapKeyName, mapValueClass);
                    hbmCol = new HbmColumn(col, null);
                }
                else
                {
                    mapKeyName = "mapKey_" + MappingUtils.getNextIndexCounter();
                    hbmCol = new HbmColumn(new Column() { Name = mapKeyName }, null);
                }
            }
            this.Items = new HbmColumn[1];
            this.Items[0] = hbmCol;
            PropertyUtils.handlePropertyType(propertyInfo, mapKeyClass, hbmCol, ref this.type, true);
        }

		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get { return Items != null ? Items.OfType<HbmColumn>() : AsColumns(); }
		}

		#endregion

		private IEnumerable<HbmColumn> AsColumns()
		{
			if (string.IsNullOrEmpty(column))
			{
				yield break;
			}
			else
			{
				yield return new HbmColumn
				{
					name = column,
					length = length,
				};
			}
		}

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get { return Items != null ? Items.OfType<HbmFormula>() : AsFormulas(); }
		}

		private IEnumerable<HbmFormula> AsFormulas()
		{
			if (string.IsNullOrEmpty(formula))
			{
				yield break;
			}
			else
			{
				yield return new HbmFormula { Text = new[] { formula } };
			}
		}

		#endregion

		#region Implementation of ITypeMapping

		public HbmType Type
		{
			get { return string.IsNullOrEmpty(type) ? null : new HbmType { name = type }; }
		}

		#endregion
	}
}