using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using System.Reflection;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmMapKeyManyToMany
	{
        public HbmMapKeyManyToMany(PropertyHelper propertyInfo, Type mapKeyClass)
            : this()
        {
            this.@class = MappingUtils.getStringClassDef(mapKeyClass);
            MapKeyJoinColumn[] mapKeyJoinColArr = propertyInfo.getAttributeArr<MapKeyJoinColumn>();
            if (mapKeyJoinColArr != null && mapKeyJoinColArr.Length > 0)
            {
                this.Items = new HbmColumn[mapKeyJoinColArr.Length];
                int i = 0;
                foreach (MapKeyJoinColumn col in mapKeyJoinColArr)
                {
                    this.Items[i] = new HbmColumn(col);
                    i++;
                }
            }
        }

		#region Implementation of IColumnsMapping

		[XmlIgnore]
		public IEnumerable<HbmColumn> Columns
		{
			get { return Items != null ? Items.OfType<HbmColumn>() : AsColumns(); }
		}

		#endregion

		private IEnumerable<HbmColumn> AsColumns()
		{
			if (string.IsNullOrEmpty(column))
			{
				yield break;
			}
			else
			{
				yield return new HbmColumn
				{
					name = column,
				};
			}
		}

		#region Implementation of IFormulasMapping

		[XmlIgnore]
		public IEnumerable<HbmFormula> Formulas
		{
			get { return Items != null ? Items.OfType<HbmFormula>() : AsFormulas(); }
		}

		private IEnumerable<HbmFormula> AsFormulas()
		{
			if (string.IsNullOrEmpty(formula))
			{
				yield break;
			}
			else
			{
				yield return new HbmFormula { Text = new[] { formula } };
			}
		}

		#endregion

		#region Implementation of IRelationship

		public string EntityName
		{
			get { return entityname; }
		}

		public string Class
		{
			get { return @class; }
		}

		public HbmNotFoundMode NotFoundMode
		{
			get { return HbmNotFoundMode.Exception; }
		}

		#endregion
	}
}