using System.Linq;
using System;
using System.Collections.Generic;
using System.Reflection;
using NPersistence.NHibernate.src.annotationSupport.mapping;
using NPersistence.NHibernate.Event;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	partial class HbmMapping 
	{
        public HbmMapping(Type entity)
            : this()
        {
            MappingCache.initColumnRegistry();
            this.defaultaccess = "property";
            List<Object> items = new List<object>();
            List<object> items1List = new List<object>();
            List<HbmResultSet> resultSetList = new List<HbmResultSet>();
            bool handled = false;
            object[] attributes = entity.GetCustomAttributes(false);

            foreach(object attr in attributes)
            {
                if (attr is Entity)
                {
                    handled = true;
                    handleEntity(entity, items);
                }
                else if (attr is NamedQuery)
                {
                    items1List.Add(new HbmQuery((NamedQuery)attr));
                }
                else if (attr is NamedNativeQuery)
                {
                    items1List.Add(new HbmSqlQuery((NamedNativeQuery)attr));
                }
                else if (attr is MappedSuperclass || attr is Embeddable)
                {
                    handled = true;
                }
            }
            if (!handled)
            {
                throw new PersistenceException("Not parsable: <" + entity.Name + ">. Are you missing a [Entity] annotation?");
            }
            handleListeners(entity);
            handleResultSet(entity, resultSetList);
            this.Items = items.ToArray();
            this.Items1 = items1List.ToArray();
            this.resultset = resultSetList.ToArray();
        }

        private void handleEntity(Type entity, List<Object> items)
        {
            Type baseType = entity.BaseType;
            if (baseType == null || baseType.Equals(typeof(Object)) || baseType.GetCustomAttributes(typeof(Entity), false).Length == 0)
            {
                addClass(entity, items, false);
            }
        }

        private void handleInheritence(Type entity, HbmClass clss, List<Object> items)
        {
            IEnumerable<Type> subclasses = MappingUtils.FindSubClassesOf(entity);
            if (subclasses == null || subclasses.Count() == 0)
            {
                return;
            }

            DiscriminatorColumn discriminatorCol;
            InheritanceType inheritenceType = MappingUtils.getInheritenceType(entity, out discriminatorCol);

            foreach (Type subclass in subclasses)
            {
                if (subclass.GetCustomAttributes(typeof(Entity), false).Length > 0)
                {
                    if (inheritenceType == InheritanceType.SINGLE_TABLE && String.IsNullOrEmpty(clss.discriminatorvalue))
                    {
                        clss.discriminatorvalue = MappingUtils.getDiscriminatorValue(entity, discriminatorCol);
                    }
                    InheritanceType childInheritenceType = inheritenceType;
                    Object[] inheritenceArr = subclass.GetCustomAttributes(typeof(Inheritance), false);
                    if (inheritenceArr.Length > 0)
                    {
                        childInheritenceType = ((Inheritance)inheritenceArr[0]).Strategy;
                    }
                    if (childInheritenceType == InheritanceType.SINGLE_TABLE)
                    {
                        if (childInheritenceType != inheritenceType)
                        {
                            object[] discriminatorArr = subclass.GetCustomAttributes(typeof(DiscriminatorColumn), false);
                            if (discriminatorArr.Length > 0)
                            {
                                discriminatorCol = (DiscriminatorColumn)discriminatorArr[0];
                            }
                        }
                        addSubclass(subclass, discriminatorCol, clss, items, false);
                    }
                    else if (childInheritenceType == InheritanceType.JOINED)
                    {
                        addSubclass(subclass, discriminatorCol, clss, items, true);
                    }
                    else // InheritanceType.TABLE_PER_CLASS
                    {
                        // inheritence of type InheritanceType.TABLE_PER_CLASS is equivalent to an ordenary HbmClass mapping that
                        // includes all inherited properties.
                        addClass(subclass, items, true);
                    }

                }
            }
        }

        private void addClass(Type entity, List<Object> items, bool includeInheritedProperties)
        {
            HbmClass hbmClass = new HbmClass(entity, includeInheritedProperties);
            items.Add(hbmClass);
            handleInheritence(entity, hbmClass, items);
        }

        private void addSubclass(Type subclass, DiscriminatorColumn discriminatorCol, HbmClass containerClass, List<Object> items, bool joined)
        {
            if (containerClass.discriminator == null)
            {
                HbmDiscriminator hbmDiscriminator = new HbmDiscriminator(discriminatorCol);
                containerClass.discriminator = hbmDiscriminator;
            }
            
            HbmSubclass hbmSub = new HbmSubclass(subclass, joined, discriminatorCol);
            items.Add(hbmSub); 
            handleInheritence(subclass, containerClass, items);
        }

        private void handleResultSet(Type entity, List<HbmResultSet> resultSetList)
        {
            object[] resultSetArr = entity.GetCustomAttributes(typeof(SqlResultSetMapping), false);
            if (resultSetArr != null && resultSetArr.Length > 0)
            {
                object[] entityResultArr = entity.GetCustomAttributes(typeof(EntityResult), false);
                object[] columnResultArr = entity.GetCustomAttributes(typeof(ColumnResult), false);
                object[] fieldResultArr = entity.GetCustomAttributes(typeof(FieldResult), false);
                foreach (object resultSetObj in resultSetArr)
                {
                    SqlResultSetMapping resultSetMapping = (SqlResultSetMapping)resultSetObj;

                    IEnumerable<EntityResult> entities =
                        from entityRes in entityResultArr
                        where ((EntityResult)entityRes).ResultSetRef.Equals(resultSetMapping.Name)
                        select (EntityResult)entityRes;

                    IEnumerable<ColumnResult> columns =
                        from columnRes in columnResultArr
                        where ((ColumnResult)columnRes).ResultSetRef.Equals(resultSetMapping.Name)
                        select (ColumnResult)columnRes;

                    resultSetList.Add(new HbmResultSet(resultSetMapping, entities, columns, fieldResultArr));
                }
            }
        }
        
        private static ISet<Type> handledTypes = new HashSet<Type>();
        
        private void handleListeners(Type entity)
        {
            if(handledTypes.Contains(entity))
            {
                //already handled
                return;
            }
            EntityListeners entityListeners = null;
            Object[] entityListenersArr = entity.GetCustomAttributes(typeof(EntityListeners), false);
            if (entityListenersArr != null && entityListenersArr.Length > 0) 
            {
                entityListeners = (EntityListeners)entityListenersArr[0];
            }
            // if this is not an entity or a mappedSuperclass - exit
            if (entity.GetCustomAttributes(typeof(Entity), false).Length == 0 && entity.GetCustomAttributes(typeof(MappedSuperclass), false).Length == 0)
            {
                if (entityListeners != null)
                {
                    throw new PersistenceException("Entity Listeners should be added only to Entity or MappedSuperclass classes. And <" + entity.Name + "> is neither.");
                }
                return;
            }
            handledTypes.Add(entity);
            // Add listeners defined on this entity/mappedsuperclass
            if (entityListeners != null && entityListeners.Value != null)
            {
                Type[] listeners = entityListeners.Value;
                foreach (Type listener in listeners)
                {
                    ConstructorInfo constructor = listener.GetConstructor(new Type[] { });
                    if (constructor == null)
                    {
                        throw new PersistenceException("A no-arg constructor is expected for EntityListener <" + listener.Name + ">.");
                    }
                    Object listenerObj = constructor.Invoke(null);
                    MethodInfo[] methods = listener.GetMethods(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly);
                    foreach (MethodInfo method in methods)
                    {
                        object[] attributes = method.GetCustomAttributes(false);
                        foreach (object attr in attributes)
                        {
                            CallbackHandler.registerCallback(attr.GetType(), entity, listenerObj, method);
                        }
                    }
                }
            }

            MethodInfo[] methodArr = entity.GetMethods(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly);
            foreach (MethodInfo method in methodArr)
            {
                object[] attributes = method.GetCustomAttributes(false);
                foreach (object attr in attributes)
                {
                    CallbackHandler.registerCallback(attr.GetType(), entity, null, method);
                }
            }
            // Add listeners from super classes (if not excluded)
            if (entity.GetCustomAttributes(typeof(ExcludeSuperclassListeners), false).Length > 0)
            {
                return;
            }
            Type baseType = entity.BaseType;
            bool isMappedSuperclass = baseType.GetCustomAttributes(typeof(MappedSuperclass), false).Length > 0;
            if (baseType.GetCustomAttributes(typeof(Entity), false).Length > 0 || isMappedSuperclass)
            {
                CallbackHandler.registerHierarchyCallback(entity, baseType);
                // Mappedsuperclass classes may not be registered in persistence.xml and never get here.
                // so - handle them now.
                if (isMappedSuperclass)
                {
                    handleListeners(baseType);
                }
            }
        }

		public HbmDatabaseObject[] DatabaseObjects
		{
			get { return databaseobject ?? new HbmDatabaseObject[0]; }
		}

		public HbmFilterDef[] FilterDefinitions
		{
			get { return filterdef ?? new HbmFilterDef[0]; }
		}

		public HbmResultSet[] ResultSets
		{
			get { return resultset ?? new HbmResultSet[0]; }
		}

		public HbmTypedef[] TypeDefinitions
		{
			get { return typedef ?? new HbmTypedef[0]; }
		}

		public HbmImport[] Imports
		{
			get { return import ?? new HbmImport[0]; }
		}

		protected HbmMeta[] Metadatas
		{
			get { return meta ?? new HbmMeta[0]; }
		}

		public HbmClass[] RootClasses
		{
			get { return Items != null ? Items.OfType<HbmClass>().ToArray():new HbmClass[0]; }
		}

		public HbmSubclass[] SubClasses
		{
			get { return Items != null ? Items.OfType<HbmSubclass>().ToArray(): new HbmSubclass[0]; }
		}

		public HbmJoinedSubclass[] JoinedSubclasses
		{
			get { return Items != null ? Items.OfType<HbmJoinedSubclass>().ToArray(): new HbmJoinedSubclass[0]; }
		}

		public HbmUnionSubclass[] UnionSubclasses
		{
			get { return Items != null ? Items.OfType<HbmUnionSubclass>().ToArray(): new HbmUnionSubclass[0]; }
		}

		public HbmQuery[] HqlQueries
		{
			get { return Items1 != null ? Items1.OfType<HbmQuery>().ToArray() : new HbmQuery[0]; }
		}

		public HbmSqlQuery[] SqlQueries
		{
			get { return Items1 != null ? Items1.OfType<HbmSqlQuery>().ToArray() : new HbmSqlQuery[0]; }
		}
	}
}