using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;
using NPersistence.Ext;
using NHibernate.Dialect;
using NPersistence.NHibernate.src.annotationSupport.mapping;

namespace NPersistence.NHibernate.annotationSupport.mapping
{
	public partial class HbmProperty : TableResident
	{
        public HbmProperty(PropertyHelper propertyInfo)
            : this()
        {
            this.name = propertyInfo.Name;
            //handle column
            PropertyUtils.handleColumn(propertyInfo, ref this.Items);
            HbmColumn hbmCol = (HbmColumn)this.Items[0];
            //handle index
            PropertyUtils.handleIndex(propertyInfo, hbmCol);
            //handle unique constraint
            PropertyUtils.handleUniqueConstrains(propertyInfo, hbmCol);
            // determine type
            PropertyUtils.handlePropertyType(propertyInfo, propertyInfo.PropertyType, hbmCol, ref this.type1, false);
            //handle basic options
            PropertyUtils.handleBasic(propertyInfo, hbmCol, ref this.lazy);
        }

        public String getTable()
        {
            if (this.Items != null && this.Items[0] != null)
            {
                return ((HbmColumn)this.Items[0]).table;
            }
            return null;
        }

        #region Implementation of IEntityPropertyMapping

        public string Name
        {
            get { return name; }
        }

        public bool IsLazyProperty
        {
            get { return lazy; }
        }

        public string Access
        {
            get { return access; }
        }

        public bool OptimisticLock
        {
            get { return optimisticlock; }
        }

        #endregion

        #region Implementation of IColumnsMapping

        [XmlIgnore]
        public IEnumerable<HbmColumn> Columns
        {
            get { return Items != null ? Items.OfType<HbmColumn>() : AsColumns(); }
        }

        #endregion

        private IEnumerable<HbmColumn> AsColumns()
        {
            if (string.IsNullOrEmpty(column))
            {
                yield break;
            }
            else
            {
                yield return new HbmColumn
                {
                    name = column,
                    length = length,
                    scale = scale,
                    precision = precision,
                    notnull = notnull,
                    notnullSpecified = notnullSpecified,
                    unique = unique,
                    uniqueSpecified = true,
                    uniquekey = uniquekey,
                    index = index,
                };
            }
        }

        #region Implementation of IFormulasMapping

        [XmlIgnore]
        public IEnumerable<HbmFormula> Formulas
        {
            get { return Items != null ? Items.OfType<HbmFormula>() : AsFormulas(); }
        }

        private IEnumerable<HbmFormula> AsFormulas()
        {
            if (string.IsNullOrEmpty(formula))
            {
                yield break;
            }
            else
            {
                yield return new HbmFormula { Text = new[] { formula } };
            }
        }

        #endregion

        #region Implementation of ITypeMapping

        public HbmType Type
        {
            get 
            { 
                HbmType ret = type ?? (!string.IsNullOrEmpty(type1) ? new HbmType { name = type1 } : null);
                Console.Out.WriteLine("###namme= " + ret.name);
                return ret;
            }
        }

        #endregion
	}
}