﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NPersistence.NHibernate.annotationSupport.mapping;
using System.Reflection;

namespace NPersistence.NHibernate.src.annotationSupport.mapping
{
    class RelationUtils
    {
        internal static void handleRelation(ref String name, PropertyHelper propertyInfo, ref Object item, ref HbmCollectionLazy lazy, ref bool lazySpecified, ref String table, ref String schema, ref String catalog, ref HbmKey key, ref Type targetType, ref String cascade, ref bool inverse, bool isIndexedCollection, ref String orderby, ref String mappedBy)
        {
            PropertyHelper inverseProperty = null;
            name = propertyInfo.Name;
            bool isManyToMany = propertyInfo.getSingleAttribute<ManyToMany>() != null;

            handleElementCollection(propertyInfo, ref item, ref lazy, ref lazySpecified, ref table, ref schema, ref catalog, ref key, ref targetType);
            handleOneToMany(propertyInfo, ref inverseProperty, ref item, ref lazy, ref lazySpecified, ref table, ref schema, ref catalog, ref key, ref targetType, ref cascade, ref inverse, isIndexedCollection, ref mappedBy);
            handleManyToMany(propertyInfo, ref inverseProperty, ref item, ref lazy, ref lazySpecified, ref table, ref schema, ref catalog, ref key, ref targetType, ref cascade, ref inverse, isIndexedCollection, ref mappedBy);
            handleJoinTable(propertyInfo, isManyToMany, ref table, ref schema, ref catalog, ref key, ref targetType);
            handleOrderBy(propertyInfo, ref orderby);
            handleTableName(isManyToMany, propertyInfo, targetType, inverse, inverseProperty, ref table);
        }

        internal static void handleElementCollection(PropertyHelper propertyInfo, ref Object item, ref HbmCollectionLazy lazy, ref bool lazySpecified, ref String table, ref String schema, ref String catalog, ref HbmKey key, ref Type targetType)
        {
            ElementCollection elementCollection = propertyInfo.getSingleAttribute<ElementCollection>();
            if (elementCollection != null)
            {
                if (elementCollection.Fetch == FetchType.EAGER)
                {
                    lazy = HbmCollectionLazy.False;
                    lazySpecified = true;
                }
                targetType = getRelationTargetEntity(elementCollection, propertyInfo);
                if (MappingUtils.isEmbeddable(targetType))
                {
                    item = new HbmCompositeElement(targetType, propertyInfo);
                }
                else
                {
                    // simple type 
                    item = new HbmElement(propertyInfo, targetType);
                }
                
                CollectionTableHelper collectionTable = MappingUtils.getCollectionTable(propertyInfo);
                if (collectionTable != null)
                {
                    table = MappingUtils.getNotEmptyString(collectionTable.table.Name);
                    schema = MappingUtils.getNotEmptyString(collectionTable.table.Schema);
                    catalog = MappingUtils.getNotEmptyString(collectionTable.table.Catalog);
                    key = new HbmKey(collectionTable, propertyInfo.ReflectedType);
                    Dictionary<String, String> uniqueConstraints = MappingUtils.getUniqueConstraints(propertyInfo, table);
                    MappingUtils.applyUniqueConstraints(uniqueConstraints, new Object[] { key, item });
                }
            }
        }

        internal static void handleOneToMany(PropertyHelper propertyInfo, ref PropertyHelper inverseProperty, ref Object item, ref HbmCollectionLazy lazy, ref bool lazySpecified, ref String table, ref String schema, ref String catalog, ref HbmKey key, ref Type targetType, ref String cascade, ref bool inverse, bool isIndexedCollection, ref String mappedBy)
        {
            OneToMany oneToMany = propertyInfo.getSingleAttribute<OneToMany>();
            if (oneToMany != null)
            {
                cascade = CascadeTypeMapping.mapCascadeType(oneToMany.Cascade, oneToMany);
                if (oneToMany.Fetch == FetchType.EAGER)
                {
                    lazy = HbmCollectionLazy.False;
                    lazySpecified = true;
                }
                
                HbmOneToMany oTom = new HbmOneToMany(propertyInfo);
                item = oTom;
                targetType = oTom.targetType;

                if (oneToMany.MappedBy != null && oneToMany.MappedBy.Length > 0)
                {
                    mappedBy = oneToMany.MappedBy;
                    handleInverse(false, propertyInfo, ref inverseProperty, oneToMany.MappedBy, ref inverse, isIndexedCollection, ref table, ref schema, ref catalog, ref key);

                    if (isIndexedCollection)
                    {
                        //For bidirectional relation and one side is an indexed collection,
                        //it is not supported by NHibernate.
                        //In this case, we put here insert=false and update=false.
                        //the indexed side of the relation is responsible for the updates.
                        RelationObject mappingRelation = MappingCache.getInstance().getRelation(targetType, oneToMany.MappedBy);
                        if (mappingRelation != null && mappingRelation is HbmManyToOne)
                        {
                            ((HbmManyToOne)mappingRelation).update = false;
                            ((HbmManyToOne)mappingRelation).insert = false;
                        }
                    }
                }
            }
        }

        internal static void handleManyToMany(PropertyHelper propertyInfo, ref PropertyHelper inverseProperty, ref Object item, ref HbmCollectionLazy lazy, ref bool lazySpecified, ref String table, ref String schema, ref String catalog, ref HbmKey key, ref Type targetType, ref String cascade, ref bool inverse, bool isIndexedCollection, ref String mappedBy)
        {
            ManyToMany manyToMany = propertyInfo.getSingleAttribute<ManyToMany>();
            if (manyToMany != null)
            {
                cascade = CascadeTypeMapping.mapCascadeType(manyToMany.Cascade, manyToMany);
                if (manyToMany.Fetch == FetchType.EAGER)
                {
                    lazy = HbmCollectionLazy.False;
                    lazySpecified = true;
                }
                if (manyToMany.MappedBy != null && manyToMany.MappedBy.Length > 0)
                {
                    mappedBy = manyToMany.MappedBy;
                    handleInverse(true, propertyInfo, ref inverseProperty, manyToMany.MappedBy, ref inverse, isIndexedCollection, ref table, ref schema, ref catalog, ref key);
                }
                HbmManyToMany mTom = new HbmManyToMany(propertyInfo, inverseProperty);
                item = mTom;
                targetType = mTom.targetType;
            }
        }

        private static void handleInverse(bool isManyToMany, PropertyHelper propertyInfo, ref PropertyHelper inverseProperty, String mappedBy, ref bool inverse, bool isIndexedCollection, ref String table, ref String schema, ref String catalog, ref HbmKey key)
        {
            inverseProperty = getRelationMappingProperty(propertyInfo, mappedBy);
            Type holdingType = propertyInfo.ReflectedType;
            if (!isIndexedCollection)
            {
                inverse = true;
            }

            JoinTableHelper joinTable = MappingUtils.getJoinTable(inverseProperty, inverseProperty.ReflectedType, holdingType);
            if (joinTable != null)
            {
                key = new HbmKey(joinTable, holdingType, isManyToMany, true);
                table = MappingUtils.getNotEmptyString(joinTable.table.Name);
                schema = MappingUtils.getNotEmptyString(joinTable.table.Schema);
                catalog = MappingUtils.getNotEmptyString(joinTable.table.Catalog);

                Dictionary<String, String> uniqueConstraints = MappingUtils.getUniqueConstraints(inverseProperty, table);
                MappingUtils.applyUniqueConstraints(uniqueConstraints, new Object[] { key });
            }
            else
            {
                JoinColumn[] joinColArr = MappingUtils.getJoinColumn(inverseProperty);
                if (joinColArr.Length == 0)
                {
                    joinColArr = new JoinColumn[] { new JoinColumn() };
                }
                for (int i = 0; i < joinColArr.Length; i++)
                {
                    joinColArr[i] = MappingUtils.setJoinColumnDefaults(joinColArr[i], inverseProperty, i);
                }
                key = new HbmKey(joinColArr, holdingType, isManyToMany, true);
                if (joinColArr != null && joinColArr.Length > 0)
                {
                    table = MappingUtils.getNotEmptyString(((JoinColumn)joinColArr[0]).Table);
                }
            }
        }

        private static PropertyHelper getRelationMappingProperty(PropertyHelper propertyInfo, string mappedBy)
        {
            String[] mappedByTokens = mappedBy.Split(new char[] { '.' }, 2);
            PropertyHelper propInfo = getTargetProperty(propertyInfo, mappedByTokens[0]);
            if (propInfo == null)
            {
                throw new ArgumentException("Could not find property: " + mappedBy);
            }
            if (mappedByTokens.Length == 1)
            {
                return propInfo;
            }
            return getRelationMappingProperty(propInfo, mappedByTokens[1]);
        }

        internal static PropertyHelper getTargetProperty(PropertyHelper propertyInfo, String property)
        {
            Type targetType = getRelationTargetEntity(null, propertyInfo);
            PropertyInfo propInfo = targetType.GetProperty(property);
            if (propInfo == null)
            {
                return null;
            }
            if (MappingUtils.isEmbeddable(targetType))
            {
                String keyOrValue = "";
                if (MappingUtils.isDictionary(propertyInfo.PropertyType))
                {
                    keyOrValue = "value";
                }
                return new PropertyHelper(propInfo, propertyInfo, keyOrValue);
            }
            return new PropertyHelper(propInfo);
        }

        internal static Type getRelationTargetEntity(Object relation, PropertyHelper propertyInfo)
        {
            if (relation == null)
            {
                object[] attributes = propertyInfo.GetCustomAttributes(false);
                foreach (object attr in attributes)
                {
                    if (attr is ElementCollection || attr is OneToMany || attr is ManyToMany || attr is ManyToOne || attr is OneToOne || attr is Embedded)
                    {
                        relation = attr;
                        break;
                    }
                }
                if (relation == null)
                {
                    throw new PersistenceException("Property '" + propertyInfo.Name + "' is missing a relation annotation.");
                }
            }

            Type targetEntity = null;
            bool isArray = false;
            if (relation is ElementCollection)
            {
                targetEntity = ((ElementCollection)relation).TargetClass;
                isArray = true;
            }
            else if (relation is OneToMany)
            {
                targetEntity = ((OneToMany)relation).TargetEntity;
                isArray = true;
            }
            else if (relation is ManyToMany)
            {
                targetEntity = ((ManyToMany)relation).TargetEntity;
                isArray = true;
            }
            else if (relation is ManyToOne)
            {
                targetEntity = ((ManyToOne)relation).TargetEntity;
            }
            else if (relation is OneToOne)
            {
                targetEntity = ((OneToOne)relation).TargetEntity;
            }

            if (targetEntity == null || targetEntity.Equals(typeof(void)))
            {
                targetEntity = propertyInfo.PropertyType;
                if (isArray)
                {
                    if (targetEntity.IsGenericType)
                    {
                        // the type is either set, list collection or map
                        Type[] args = targetEntity.GetGenericArguments();
                        targetEntity = args[args.Length - 1];
                    }
                    else
                    {
                        throw new ArgumentException("TargetEntity is missing. For a relation without generics notation, the TargetEntity property is mandatory");
                    }
                }
            }
            return targetEntity;
        }

        internal static void handleJoinTable(PropertyHelper propertyInfo, bool isManyToMany, ref String table, ref String schema, ref String catalog, ref HbmKey key, ref Type targetType)
        {
            JoinTableHelper joinTable = MappingUtils.getJoinTable(propertyInfo, propertyInfo.ReflectedType, targetType);
            if (joinTable != null)
            {
                table = MappingUtils.getNotEmptyString(joinTable.table.Name);
                schema = MappingUtils.getNotEmptyString(joinTable.table.Schema);
                catalog = MappingUtils.getNotEmptyString(joinTable.table.Catalog);
                key = new HbmKey(joinTable, propertyInfo.ReflectedType, isManyToMany, false);

                Dictionary<String, String> uniqueConstraints = MappingUtils.getUniqueConstraints(propertyInfo, table);
                MappingUtils.applyUniqueConstraints(uniqueConstraints, new Object[]{ key});
            }
            // if this is an inverse relation, key should have been populated by now. else, handle it now.
            if (key == null)
            {
                JoinColumn[] attrArr = MappingUtils.getJoinColumn(propertyInfo);
                if (attrArr == null || attrArr.Length == 0)
                {
                    attrArr = new JoinColumn[] { new JoinColumn() };
                }
                for (int i = 0; i < attrArr.Length; i++)
                {
                    attrArr[i] = MappingUtils.setJoinColumnDefaults(attrArr[i], propertyInfo, i);
                }
                key = new HbmKey(attrArr, propertyInfo.ReflectedType, isManyToMany, false);
            }
        }

        internal static void handleOrderBy(PropertyHelper propertyInfo, ref String orderby)
        {
            OrderBy ordBy = propertyInfo.getSingleAttribute<OrderBy>();
            if (ordBy != null)
            {
                orderby = ordBy.Value;
            }
        }

        internal static void handleTableName(bool isManyToMany, PropertyHelper propertyInfo, Type targetType, bool inverse, PropertyHelper inverseProperty, ref String table)
        {
            if (!isManyToMany || table != null)
            {
                return;
            }

            if (inverse)
            {
                table = MappingUtils.getJoinTableName(inverseProperty.ReflectedType, propertyInfo.ReflectedType);
            }
            else
            {
                table = MappingUtils.getJoinTableName(propertyInfo.ReflectedType, targetType);
            }
        }

    }
}
