﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NPersistence.spi;

namespace NPersistence.NHibernate.packaging
{
    /**
    * Simple represenation of persistence.xml
    */
    public class PersistenceMetadata
    {
        private String m_name;
        private String m_nonJtaDatasource;
        private String m_jtaDatasource;
        private String m_provider;
        private PersistenceUnitTransactionType m_transactionType = PersistenceUnitTransactionType.RESOURCE_LOCAL;
        private List<String> m_classes = new List<String>();
        private List<String> m_packages = new List<String>();
        private List<String> m_mappingFiles = new List<String>();
        private ISet<String> m_jarFiles = new HashSet<String>();
        private List<NamedInputStream> m_hbmfiles = new List<NamedInputStream>();
        private Dictionary<string, string> m_props = new Dictionary<string, string>();
        private bool m_excludeUnlistedClasses = false;

        public String getName()
        {
            return m_name;
        }

        public void setName(String name)
        {
            this.m_name = name;
        }

        public PersistenceUnitTransactionType getTransactionType()
        {
            return m_transactionType;
        }

        public void setTransactionType(PersistenceUnitTransactionType transactionType)
        {
            this.m_transactionType = transactionType;
        }

        public String getNonJtaDatasource()
        {
            return m_nonJtaDatasource;
        }

        public void setNonJtaDatasource(String nonJtaDatasource)
        {
            this.m_nonJtaDatasource = nonJtaDatasource;
        }

        public String getJtaDatasource()
        {
            return m_jtaDatasource;
        }

        public void setJtaDatasource(String jtaDatasource)
        {
            this.m_jtaDatasource = jtaDatasource;
        }

        public String getProvider()
        {
            return m_provider;
        }

        public void setProvider(String provider)
        {
            if (provider != null && provider.EndsWith(".class"))
            {
                this.m_provider = provider.Substring(0, provider.Length - 6);
            }
            this.m_provider = provider;
        }

        public List<String> getClasses()
        {
            return m_classes;
        }

        public void setClasses(List<String> classes)
        {
            this.m_classes = classes;
        }

        public List<String> getPackages()
        {
            return m_packages;
        }

        public void setPackages(List<String> packages)
        {
            this.m_packages = packages;
        }

        public List<String> getMappingFiles()
        {
            return m_mappingFiles;
        }

        public void setMappingFiles(List<String> mappingFiles)
        {
            this.m_mappingFiles = mappingFiles;
        }

        public ISet<String> getJarFiles()
        {
            return m_jarFiles;
        }

        public void setJarFiles(ISet<String> jarFiles)
        {
            this.m_jarFiles = jarFiles;
        }

        public Dictionary<string, string> getProps()
        {
            return m_props;
        }

        public void setProps(Dictionary<string, string> props)
        {
            this.m_props = props;
        }

        public List<NamedInputStream> getHbmfiles()
        {
            return m_hbmfiles;
        }

        /**
         * @deprecated use getHbmfiles() rather
         */
        public void setHbmfiles(List<NamedInputStream> hbmfiles)
        {
            this.m_hbmfiles = hbmfiles;
        }

        public bool getExcludeUnlistedClasses()
        {
            return m_excludeUnlistedClasses;
        }

        public void setExcludeUnlistedClasses(bool excludeUnlistedClasses)
        {
            this.m_excludeUnlistedClasses = excludeUnlistedClasses;
        }


        public String toString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("PersistenceMetadata [\n")
                    .Append("\tname: ").Append(m_name).Append("\n")
                    .Append("\tjtaDataSource: ").Append(m_jtaDatasource).Append("\n")
                    .Append("\tnonJtaDataSource: ").Append(m_nonJtaDatasource).Append("\n")
                    .Append("\ttransactionType: ").Append(m_transactionType).Append("\n")
                    .Append("\tprovider: ").Append(m_provider).Append("\n")
                    .Append("\tclasses[\n");
            if (m_classes != null)
            {
                foreach (String elt in m_classes)
                {
                    sb.Append("\t\t").Append(elt);
                }
            }
            sb.Append("\t]\n")
                    .Append("\tpackages[\n");
            if (m_packages != null)
            {
                foreach (String elt in m_packages)
                {
                    sb.Append("\t\t").Append(elt).Append("\n");
                }
            }
            sb.Append("\t]\n")
                    .Append("\tmappingFiles[\n");
            if (m_mappingFiles != null)
            {
                foreach (String elt in m_mappingFiles)
                {
                    sb.Append("\t\t").Append(elt).Append("\n");
                }
            }
            sb.Append("\t]\n")
                    .Append("\tjarFiles[\n");
            if (m_jarFiles != null)
            {
                foreach (String elt in m_jarFiles)
                {
                    sb.Append("\t\t").Append(elt).Append("\n");
                }
            }
            sb.Append("\t]\n")
                    .Append("\thbmfiles: ")
                    .Append(m_hbmfiles != null ? m_hbmfiles.Count() : 0).Append("\n")
                    .Append("\tDictionary[\n");

            if (m_props != null)
            {
                foreach(KeyValuePair<string, string> pair in m_props)
                {
                    sb.Append("\t\t").Append(pair.Key).Append(": ").Append(pair.Value).Append("\n");
                }
            }
            sb.Append("\t]").Append("]");

            return sb.ToString();
        }
    }
}
