using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using log4net;
using NPersistence.spi;
using System.Xml;
using NPersistence.NHibernate.util;
using NPersistence;

namespace NPersistence.NHibernate.packaging
{

    public class PersistenceXmlLoader 
    {
	    private static ILog logger = LogManager.GetLogger("NPersistence.NHibernate.packaging");

        public static List<PersistenceMetadata> deploy(System.IO.Stream stream, Dictionary<object, object> overrides) 
        {
            List<PersistenceMetadata> units = new List<PersistenceMetadata>();
            XmlDocument doc = new XmlDocument();
            doc.Load(stream);
            XmlNodeList nodeList = doc.GetElementsByTagName(XmlHelper.PERSISTENCE_UNIT);
            IEnumerator nodesEnum = nodeList.GetEnumerator();
            while(nodesEnum.MoveNext())
            {
                XmlNode element = (XmlNode)nodesEnum.Current;
				PersistenceMetadata metadata = parsePersistenceUnit( element );
                applyOverrides(metadata, overrides);
			    units.Add( metadata );
			}
		    return units;
        }

        public static List<PersistenceMetadata> webDeploy(Dictionary<object, object> overrides)
        {
            List<PersistenceMetadata> units = new List<PersistenceMetadata>();
            System.Xml.XmlNode persistenceUnit = null;
            try
            {
                persistenceUnit = (System.Xml.XmlNode)System.Configuration.ConfigurationManager.GetSection(XmlHelper.PERSISTENCE_UNIT);
            }
            catch (Exception e)
            {
                if (logger.IsErrorEnabled) logger.Error("failed to get configuration from web.config file");
            }

            if (persistenceUnit != null)
            {
                PersistenceMetadata metadata = parsePersistenceUnit(persistenceUnit);
                applyOverrides(metadata, overrides);
                units.Add(metadata);
            }
            return units;
        }

        internal static void applyOverrides(PersistenceMetadata metadata, Dictionary<object, object> overrides)
        {
            //override properties of metadata if needed
            if (overrides.ContainsKey(AvailableSettings.PROVIDER))
            {
                String provider = (String)overrides[AvailableSettings.PROVIDER];
                metadata.setProvider(provider);
            }
            else if (overrides.ContainsKey(AvailableSettings.PROVIDER_JPA))
            {
                String provider = (String)overrides[AvailableSettings.PROVIDER_JPA];
                metadata.setProvider(provider);
            }

            if (overrides.ContainsKey(AvailableSettings.TRANSACTION_TYPE))
            {
                String transactionType = (String)overrides[AvailableSettings.TRANSACTION_TYPE];
                metadata.setTransactionType(PersistenceXmlLoader.getTransactionType(transactionType).Value);
            }
            else if (overrides.ContainsKey(AvailableSettings.TRANSACTION_TYPE_JPA))
            {
                String transactionType = (String)overrides[AvailableSettings.TRANSACTION_TYPE_JPA];
                metadata.setTransactionType(PersistenceXmlLoader.getTransactionType(transactionType).Value);
            }
            if (overrides.ContainsKey(AvailableSettings.JTA_DATASOURCE))
            {
                String dataSource = (String)overrides[AvailableSettings.JTA_DATASOURCE];
                metadata.setJtaDatasource(dataSource);
            }
            else if (overrides.ContainsKey(AvailableSettings.JTA_DATASOURCE_JPA))
            {
                String dataSource = (String)overrides[AvailableSettings.JTA_DATASOURCE_JPA];
                metadata.setJtaDatasource(dataSource);
            }

            if (overrides.ContainsKey(AvailableSettings.NON_JTA_DATASOURCE))
            {
                String dataSource = (String)overrides[AvailableSettings.NON_JTA_DATASOURCE];
                metadata.setNonJtaDatasource(dataSource);
            }

            else if (overrides.ContainsKey(AvailableSettings.NON_JTA_DATASOURCE_JPA))
            {
                String dataSource = (String)overrides[AvailableSettings.NON_JTA_DATASOURCE_JPA];
                metadata.setNonJtaDatasource(dataSource);
            }

            /*
             * if explicit => use it
             * if JTA DS => JTA transaction
             * if non JTA DA => RESOURCE_LOCAL transaction
             * else default JavaSE => RESOURCE_LOCAL
             */
            PersistenceUnitTransactionType trnsType = metadata.getTransactionType();
            if (trnsType == PersistenceUnitTransactionType.JTA)
            {
                throw new NotSupportedException("JTA is not supported");
            }
            /*Boolean? isJTA = null;
                    
            if ( metadata.getJtaDatasource() != null && metadata.getJtaDatasource().Length > 0 ) 
            {
                isJTA = true;
            }
            else if ( metadata.getNonJtaDatasource() != null && metadata.getNonJtaDatasource().Length > 0) 
            {
                isJTA = false;
            }

            if (trnsType == null) 
            {
                if(!isJTA.HasValue)
                {
                    trnsType = defaultTransactionType;
                }
                else
                {
                    if ((bool)isJTA) 
                    {
                        //trnsType = PersistenceUnitTransactionType.JTA;
                        throw new NotSupportedException("JTA is not supported");
                    }
                    else
                    {
                        trnsType = PersistenceUnitTransactionType.RESOURCE_LOCAL;
                    }
                }
            }
            metadata.setTransactionType( trnsType );
            */
            Dictionary<string, string> properties = metadata.getProps();
            ConfigurationHelper.overrideProperties(properties, overrides);
        }
        

	    internal static PersistenceMetadata parsePersistenceUnit(XmlNode top)
        {
		    PersistenceMetadata metadata = new PersistenceMetadata();
            XmlAttributeCollection attributes = top.Attributes;
            Dictionary<string, string> attributesMap = XmlHelper.getAttributeMapping(attributes);
		    
            String puName;
            attributesMap.TryGetValue("name", out puName);
    		if ( puName != null && puName.Length > 0 ) 
            {
                if(logger.IsDebugEnabled)
                {
                    logger.Debug("Persistent Unit name from persistence.xml: " + puName );
                }			    
			    metadata.setName( puName );
		    }
		    XmlNodeList children = top.ChildNodes;
		    foreach ( XmlNode childNode in children) 
            {
			    if ( childNode.NodeType == XmlNodeType.Element ) 
                {
				
				    String tag = childNode.Name;

                    //if ( tag.Equals( "non-jta-data-source" ) ) 
                    //{
                    //    metadata.setNonJtaDatasource( XmlHelper.getElementContent( childNode ) );
                    //}
                    //else if ( tag.Equals( "jta-data-source" ) ) 
                    //{
                    //    metadata.setJtaDatasource( XmlHelper.getElementContent( childNode ) );
                    //}
				    if ( tag.Equals( "provider" ) ) 
                    {
					    metadata.setProvider( XmlHelper.getElementContent( childNode ) );
				    }
				    else if ( tag.Equals( "class" ) ) 
                    {
					    metadata.getClasses().Add( XmlHelper.getElementContent( childNode ) );
				    }
				    else if ( tag.Equals( "mapping-file" ) ) 
                    {
					    metadata.getMappingFiles().Add( XmlHelper.getElementContent( childNode ) );
				    }
				    else if ( tag.Equals( "jar-file" ) ) 
                    {
                        metadata.getJarFiles().Add( XmlHelper.getElementContent( childNode ) );
                    }
				    else if ( tag.Equals( "exclude-unlisted-classes" ) ) 
                    {
					    metadata.setExcludeUnlistedClasses( true );
				    }
				    else if ( tag.Equals( "properties" ) ) 
                    {
                        XmlNodeList props = childNode.ChildNodes;
		                foreach ( XmlNode propertyNode in props) 
                        {
			                if ( childNode.NodeType == XmlNodeType.Element ) 
                            {
						        if ( !"property".Equals( propertyNode.Name) ) continue;

                                Dictionary<string, string> propertyAttr = XmlHelper.getAttributeMapping(propertyNode.Attributes);

                                if (!propertyAttr.ContainsKey("name"))
                                {
                                    throw new PersistenceException("The attribute 'name' is missing from the XML node");
                                }
							    String propName = propertyAttr["name"].Trim();
                                String propValue;
                                if (propertyAttr.ContainsKey("value"))
                                {
                                    propValue = propertyAttr["value"].Trim();
                                }
                                else
                                {
                                    propValue = XmlHelper.getElementContent(propertyNode);
                                }
                                
							    
							    if ( propValue == null || propValue.Length <= 0 ) 
                                {
    								//fall back to the natural (Hibernate) way of description
								    propValue = XmlHelper.getElementContent( propertyNode, "" );
							    }
							    metadata.getProps().Add( propName, propValue );
						    }
                        }
					}
				}
			}

            if (attributesMap.ContainsKey("transaction-type"))
            {
                PersistenceUnitTransactionType? transactionType = getTransactionType(attributesMap["transaction-type"]);
                if (transactionType.HasValue)
                {
                    metadata.setTransactionType(transactionType.Value);
                }
            }
		    

    		return metadata;
        }


	public static PersistenceUnitTransactionType? getTransactionType(String elementContent) 
    {
		if ( elementContent == null || elementContent.Length <= 0) 
        {
            return null; //PersistenceUnitTransactionType.RESOURCE_LOCAL;
		}
		else if ( elementContent.ToUpper().Equals( "JTA" ) ) 
        {
			return PersistenceUnitTransactionType.JTA;
		}
		else if ( elementContent.ToUpper().Equals( "RESOURCE_LOCAL" ) ) 
        {
			return PersistenceUnitTransactionType.RESOURCE_LOCAL;
		}
		else
        {
			throw new PersistenceException( "Unknown TransactionType: " + elementContent );
		}
	}

    //public static class ErrorLogger implements ErrorHandler {
    //    private String file;
    //    private List errors;
    //    private EntityResolver resolver;

    //    ErrorLogger(String file, List errors, EntityResolver resolver) {
    //        this.file = file;
    //        this.errors = errors;
    //        this.resolver = resolver;
    //    }

    //    public void error(SAXParseException error) {
    //        if ( resolver instanceof EJB3DTDEntityResolver ) {
    //            if ( ( (EJB3DTDEntityResolver) resolver ).isResolved() == false ) return;
    //        }
    //        log.error( "Error parsing XML: {}({}) {}", new Object[] { file, error.getLineNumber(), error.getMessage() } );
    //        errors.add( error );
    //    }

    //    public void fatalError(SAXParseException error) {
    //        log.error( "Error parsing XML: {}({}) {}", new Object[] { file, error.getLineNumber(), error.getMessage() } );
    //        errors.add( error );
    //    }

    //    public void warning(SAXParseException warn) {
    //        log.warn( "Warning parsing XML: {}({}) {}", new Object[] { file, warn.getLineNumber(), warn.getMessage() } );
    //    }
    //}

}
}
