﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate;
using NPersistence;

namespace NPersistence.NHibernate.util
{
    class CacheModeHelper
    {
        public const CacheMode DEFAULT_LEGACY_MODE = CacheMode.Normal;
	    public const CacheStoreMode DEFAULT_STORE_MODE = CacheStoreMode.USE;
        public const CacheRetrieveMode DEFAULT_RETRIEVE_MODE = CacheRetrieveMode.USE;

        /**
	 * Given a JPA {@link CacheStoreMode} and {@link CacheRetrieveMode}, determine the corresponding
	 * legacy Hibernate {@link CacheMode}.
	 *
	 * @param storeMode The JPA shared-cache store mode.
	 * @param retrieveMode The JPA shared-cache retrieve mode.
	 *
	 * @return Corresponding {@link CacheMode}.
	 */
        public static CacheMode interpretCacheMode(CacheStoreMode? storeMode, CacheRetrieveMode? retrieveMode) 
        {
		    if ( !storeMode.HasValue ) 
            {
			    storeMode = DEFAULT_STORE_MODE;
		    }
		    if ( retrieveMode == null ) 
            {
			    retrieveMode = DEFAULT_RETRIEVE_MODE;
		    }

		    bool get = ( CacheRetrieveMode.USE == retrieveMode );

		    switch ( storeMode ) 
            {
                case CacheStoreMode.USE:
                {
				    return get ? CacheMode.Normal : CacheMode.Put;
			    }
                case CacheStoreMode.REFRESH:
                    {
				    // really (get == true) here is a bit of an invalid combo...
				    return CacheMode.Refresh;
			    }
                case CacheStoreMode.BYPASS:
                    {
				        return get ? CacheMode.Get : CacheMode.Ignore;
			    }
			    default: 
                    {
                        throw new InvalidOperationException("Illegal StoreMode: " + storeMode);
			    }
		    }
	    }

        public static CacheStoreMode interpretCacheStoreMode(CacheMode? cacheMode)
        {
            if (cacheMode == null)
            {
                cacheMode = DEFAULT_LEGACY_MODE;
            }

            if (CacheMode.Refresh == cacheMode)
            {
                return CacheStoreMode.REFRESH;
            }
            if (CacheMode.Normal == cacheMode || CacheMode.Put == cacheMode)
            {
                return CacheStoreMode.USE;
            }
            return CacheStoreMode.BYPASS;
        }

        public static CacheRetrieveMode interpretCacheRetrieveMode(CacheMode? cacheMode)
        {
            if (cacheMode == null)
            {
                cacheMode = DEFAULT_LEGACY_MODE;
            }

            return (CacheMode.Normal == cacheMode || CacheMode.Get == cacheMode)
                    ? CacheRetrieveMode.USE
                    : CacheRetrieveMode.BYPASS;
        }
    }
}
