using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using log4net;
using NPersistence.spi;
using System.Xml;
using NHibernate;
using NPersistence;

namespace NPersistence.NHibernate.util
{
    public class ConfigurationHelper
    {
        public static void overrideProperties(Dictionary<string, string> properties, Dictionary<object, object> overrides) 
        {
		foreach ( KeyValuePair<object, object> pair in overrides ) 
        {
			if ( pair.Key is String && pair.Value is String ) 
            {
				properties.Add((string)pair.Key, (string)pair.Value);
			}
		}
	}

        public static FlushMode? getFlushMode(Object value) 
        {
            if (value == null)
            {
                return null;
            }

            FlushMode? flushMode = null;
            if (value is FlushMode) 
            {
                flushMode = (FlushMode) value;
            }
            else if (value is FlushModeType) 
            {
                flushMode = ConfigurationHelper.getFlushMode( (FlushModeType) value);
            }
            else if (value is String) 
            {
                flushMode = ConfigurationHelper.getFlushMode( (String) value);
            }
            if (flushMode == null) 
            {
                throw new PersistenceException("Unable to parse org.hibernate.flushMode: " + value);
            }
            return flushMode;
        }

        private static FlushMode? getFlushMode(String flushMode)
        {
            if (flushMode == null) return null;
            flushMode = flushMode.ToLower();
            if (flushMode.Equals("always"))
            {
                return FlushMode.Always;
            }
            else if (flushMode.Equals("auto"))
            {
                return FlushMode.Auto;
            }
            else if (flushMode.Equals("commit"))
            {
                return FlushMode.Commit;
            }
            else if (flushMode.Equals("never"))
            {
                return FlushMode.Never;
            }
            else if (flushMode.Equals("unspecified"))
            {
                return FlushMode.Unspecified;
            }
            return null;
        }

        private static FlushMode getFlushMode(FlushModeType flushMode)
        {
            switch (flushMode)
            {
                case FlushModeType.AUTO:
                    return FlushMode.Auto;
                case FlushModeType.COMMIT:
                    return FlushMode.Commit;
                default:
                    throw new AssertionFailure("Unknown FlushModeType: " + flushMode);
            }

        }

        public static int getInteger(Object value)
        {
            if (value is int)
            {
                return (int)value;
            }
            else
            {
                return int.Parse((String)value);
            }
        }

        public static Boolean getBoolean(Object value) 
        {
            if ( value is Boolean ) 
            {
                return (Boolean) value;
            }
            else 
            {
                return Boolean.Parse((String)value );
            }
        }

        public static CacheMode getCacheMode(Object value) 
        {
            if ( value is CacheMode ) 
            {
                return (CacheMode) value;
            }
            else 
            {
                String cacheMode = (String)value;
                if (cacheMode.ToLower().Equals("get"))
                {
                    return CacheMode.Get;
                }
                else if (cacheMode.ToLower().Equals("ignore"))
                {
                    return CacheMode.Ignore;
                }
                else if (cacheMode.ToLower().Equals("normal"))
                {
                    return CacheMode.Normal;
                }
                else if (cacheMode.ToLower().Equals("put"))
                {
                    return CacheMode.Put;
                }
                else if (cacheMode.ToLower().Equals("refresh"))
                {
                    return CacheMode.Refresh;
                }
                throw new Exception("Not a legal cache mode: " + cacheMode);
            }
        }

        internal static CacheRetrieveMode getCacheRetrieveMode(object value)
        {
            if(value == null)
            {
                return default(CacheRetrieveMode);
            }
            if (value is CacheRetrieveMode)
            {
                return (CacheRetrieveMode)value;
            }
            if (value is String)
            { 
                if( ((String)value).ToLower().Equals(CacheRetrieveMode.USE.ToString().ToLower()))
                {
                    return CacheRetrieveMode.USE;
                }
                else if (((String)value).ToLower().Equals(CacheRetrieveMode.BYPASS.ToString().ToLower()))
                {
                    return CacheRetrieveMode.BYPASS;
                }
            }
            throw new Exception("Not a legal CacheRetrieveMode: " + value);
        }

        internal static CacheStoreMode getCacheStoreMode(object value)
        {
            if (value == null)
            {
                return default(CacheStoreMode);
            }
            if (value is CacheStoreMode)
            {
                return (CacheStoreMode)value;
            }
            if (value is String)
            {
                if (((String)value).ToLower().Equals(CacheStoreMode.BYPASS.ToString().ToLower()))
                {
                    return CacheStoreMode.BYPASS;
                }
                else if (((String)value).ToLower().Equals(CacheStoreMode.REFRESH.ToString().ToLower()))
                {
                    return CacheStoreMode.REFRESH;
                }
                else if (((String)value).ToLower().Equals(CacheStoreMode.USE.ToString().ToLower()))
                {
                    return CacheStoreMode.USE;
                }
            }
            throw new Exception("Not a legal CacheStoreMode: " + value);
        }
    }
}
